import sqlite3 from "sqlite3";
import fs from "fs";

const dbDirectory = "./db";
const dbFilePath = `${dbDirectory}/database.db`;

// Fonctions utilitaires pour les requêtes SQL

function runDB(
  db: sqlite3.Database,
  query: string,
  params: any[]
): Promise<void> {
  return new Promise((resolve, reject) => {
    db.run(query, params, (err) => {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

function allDB(db: sqlite3.Database, query: string): Promise<any[]> {
  return new Promise((resolve, reject) => {
    db.all(query, (err, rows) => {
      if (err) {
        reject(err);
      } else {
        resolve(rows);
      }
    });
  });
}

function getDB(
  db: sqlite3.Database,
  query: string,
  params: any[]
): Promise<void> {
  return new Promise((resolve, reject) => {
    db.get(query, params, (err, row) => {
      if (err) {
        reject(err);
      } else {
        resolve();
      }
    });
  });
}

// Fonctions pour la gestion de la base de données

// Créer le répertoire db s'il n'existe pas
function createDbDirectory() {
  if (!fs.existsSync(dbDirectory)) {
    fs.mkdirSync(dbDirectory);
  }
}

// Ouvrir la base de données
function openDatabase() {
  console.log("Ouverture de la connexion à la base de données.");
  return new sqlite3.Database(
    dbFilePath,
    sqlite3.OPEN_READWRITE | sqlite3.OPEN_CREATE,
    (err: Error | null) => {
      if (err) console.error(err.message);
    }
  );
}

// Fermer la base de données
function closeDatabase(db: sqlite3.Database) {
  db.close((err) => {
    if (err) {
      console.error(err.message);
    }
    console.log("Fermeture de la connexion à la base de données.");
  });
}

/////////////////////////// Gestion des utilisateurs ///////////////////////////
// CREATE TABLE registered_user (
//      id_user SERIAL PRIMARY KEY,
//      login VARCHAR(64) NOT NULL,
//      password VARCHAR(72) NOT NULL,
//      permissions INT NOT NULL,
//      UNIQUE (login)
// );

// Créer la table registered_user dans la base de données
function createRegisteredUserTable(db: sqlite3.Database) {
  const tableRegisteredUser = `CREATE TABLE IF NOT EXISTS registered_user (id_user INTEGER PRIMARY KEY AUTOINCREMENT, login TEXT NOT NULL, password TEXT NOT NULL, permissions INTEGER NOT NULL, UNIQUE (login))`;

  db.run(tableRegisteredUser, [], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log("Table créée avec succès");
  });
}

// Insérer un utilisateur dans la table registered_user
function insertUser(
  db: sqlite3.Database,
  login: string,
  password: string,
  permissions: number
) {
  const insertUserQuery = `INSERT INTO registered_user (login, password, permissions) VALUES (?, ?, ?)`;

  db.run(insertUserQuery, [login, password, permissions], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(
      `Utilisateur inséré avec succès. ID utilisateur: ${this.lastID}`
    );
  });
}

// Modifier le login d'un utilisateur dans la table registered_user
function updateUserLogin(db: sqlite3.Database, id: number, newLogin: string) {
  const updateUserLoginQuery = `UPDATE registered_user SET login = ? WHERE id_user = ?`;
  db.run(updateUserLoginQuery, [newLogin, id], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(`Login de l'utilisateur mis à jour avec succès.`);
  });
}

// Modifier le mot de passe d'un utilisateur dans la table registered_user
function updateUserPassword(
  db: sqlite3.Database,
  id: number,
  newPassword: string
) {
  const updateUserPasswordQuery = `UPDATE registered_user SET password = ? WHERE id_user = ?`;

  db.run(updateUserPasswordQuery, [newPassword, id], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(`Mot de passe de l'utilisateur mis à jour avec succès.`);
  });
}

function updateUserPermissions(
  db: sqlite3.Database,
  id: number,
  newPermissions: number
) {
  const updateUserPermissionsQuery = `UPDATE registered_user SET permissions = ? WHERE id_user = ?`;

  db.run(updateUserPermissionsQuery, [newPermissions, id], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(`Permissions de l'utilisateur mises à jour avec succès.`);
  });
}

// Supprimer un utilisateur de la table registered_user
function deleteUser(db: sqlite3.Database, id: number) {
  const deleteUserQuery = `DELETE FROM registered_user WHERE id_user = ?`;

  db.run(deleteUserQuery, [id], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(`Utilisateur supprimé avec succès.`);
  });
}

// Supprimer tous les utilisateurs de la table registered_user
function deleteAllUsers(db: sqlite3.Database) {
  const deleteAllUsersQuery = `DELETE FROM registered_user`;

  db.run(deleteAllUsersQuery, [], function (err) {
    if (err) {
      return console.error(err.message);
    }
    console.log(`Tous les utilisateurs ont été supprimés avec succès.`);
  });
}

// Sélectionner tous les utilisateurs de la table registered_user
function selectAllUsers(db: sqlite3.Database): Promise<unknown[]> {
  const selectAllUsersQuery = `SELECT * FROM registered_user`;

  return new Promise((resolve, reject) => {
    db.all(selectAllUsersQuery, (err, rows) => {
      if (err) {
        reject(err);
      } else {
        console.log("Utilisateurs :");
        console.log(rows);
        resolve(rows);
      }
    });
  });
}

// Sélectionner un utilisateur par son login
function selectUserByLogin(db: sqlite3.Database, login: string) {
  const selectUserByLoginQuery = `SELECT * FROM registered_user WHERE login = ?`;

  db.get(selectUserByLoginQuery, [login], (err, row) => {
    console.log("Utilisateur avec le login, " + login + " :");
    if (err) {
      return console.error(err.message);
    }
    console.log(row);
  });
}

// Sélectionner un utilisateur par son ID
function selectUserById(db: sqlite3.Database, id: number) {
  const selectUserByIdQuery = `SELECT * FROM registered_user WHERE id_user = ?`;

  db.get(selectUserByIdQuery, [id], (err, row) => {
    console.log("Utilisateur avec l'ID, " + id + " :");
    if (err) {
      return console.error(err.message);
    }
    console.log(row);
  });
}

/////////////////////////// Export des fonctions ///////////////////////////
export {
  createDbDirectory,
  openDatabase,
  closeDatabase,
  createRegisteredUserTable,
  insertUser,
  updateUserLogin,
  updateUserPassword,
  updateUserPermissions,
  deleteUser,
  deleteAllUsers,
  selectAllUsers,
  selectUserByLogin,
  selectUserById,
};
